﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;

/*
 * 2012.09.28 
 * C# Sample Code  
 * - TCP Asynchronous client
 * Sollae Systems 
 * Lisa Shin
*/

namespace TCPClientApplication
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            // form intializing 
            InitializeComponent();

            /* variables 
             SendBtn = send button 
             DisconnBtn = disconnect button 
             ConnBtn = connect button 
             ip = ip text box
             port = port text box
            */
            SendBtn.Enabled = false;
            DisconnBtn.Enabled = false;
            ip.Select();            
        }

        // A socekt of Client 
        Socket clientsock;
        // A buffer of transmitted
        byte[] MsgSendBuff  = new byte[1024];
        // A buffer of received  
        byte[] MsgRecvBuff = new byte[1024];

        // Connect button click event 
        private void ConnBtn_Click(object sender, EventArgs e)
        {
            string strIP;
            string strport;
            int port;

            // Get the IP text value
            strIP = this.ip.Text;
            // Get the port text value
            strport = this.port.Text;
            port = Int32.Parse(strport);

             
            IPAddress peerIP = IPAddress.Parse(strIP);
            // There are IP or port information in the localEP
            IPEndPoint peerEP = new IPEndPoint(peerIP, port);

            // Client socket memory allocation
            clientsock = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP);
            
            this.statText.Text = "Connecting.. \n";

            try
            {
                // Send the signal for connect with server 
                // This application will be run asynchronously by this function.
                // If the client accept the socekt, this function called a ConnectCallback function.
                clientsock.BeginConnect(peerEP, new AsyncCallback(ConnectCallback), clientsock) ;
     
                this.ConnBtn.Enabled = false;
                this.DisconnBtn.Enabled = true;
                this.SendBtn.Enabled = true;

                
            }
            catch (SocketException er)
            {
                // If the socket exception is occurrence, the application display a error message.  
                MessageBox.Show(er.Message);
            }
            
        }

        // BeginConnect()'s call back function 
        public void ConnectCallback(IAsyncResult ar)
        {
            try
            {
                // If the client socket connect with the server, return the socket
                clientsock = (Socket)ar.AsyncState;
                clientsock.EndConnect(ar);
                
                this.statText.Text = "Connected \n";

                // If the client transmitted the messages, this function called a Callback_ReceiveMsg function.
                clientsock.BeginReceive(MsgRecvBuff, 0, MsgRecvBuff.Length, SocketFlags.None, new AsyncCallback(CallBack_ReceiveMsg), clientsock);
            }
            catch (SocketException er)
            {
                MessageBox.Show(er.Message);
                DisconnBtn.Enabled = false;
                ConnBtn.Enabled = true;
                SendBtn.Enabled = false;
            }

        }

        // BeginReceive()'s call back fucntion
        public void CallBack_ReceiveMsg(IAsyncResult ar)
        {

            int length;
            // A variable of decimal message
            String MsgRecvStr = null;
            // A variable of hex message
            String MsgRecvHexStr = null;

            try
            {
                // If the socket close, return the 0 value 
                length = clientsock.EndReceive(ar);

                if (length > 0)
                {

                    // display with decimal data
                    MsgRecvStr = Encoding.Default.GetString(MsgRecvBuff, 0, length);

                    // display with hex data
                    MsgRecvHexStr = ToHexString(MsgRecvStr, "Recv");

                    this.ConnText.AppendText("Received Data : " + MsgRecvHexStr + " | " + MsgRecvStr + "\n");
                    this.ConnText.ScrollToCaret();

                    // The socket can continuedly wait the receive message because of this function 
                    clientsock.BeginReceive(MsgRecvBuff, 0, MsgRecvBuff.Length, SocketFlags.None, new AsyncCallback(CallBack_ReceiveMsg), clientsock);

                }
                else
                {
                    // If the socket close, return the 0 value 
                    // Then, socket close.
                    this.statText.Text = "Closed\n";
                    DisconnBtn.Enabled = false;
                    SendBtn.Enabled = false;
                    clientsock.Close();
                    ConnBtn.Enabled = true;
                }

            }
            catch (Exception er)
            {
                MessageBox.Show(er.Message);
            }
        }
       
        // Send button click event
        private void SendBtn_Click(object sender, EventArgs e)
        {
            String tmp;
            int sendLength;
            int i = 0;

            try
            {

                // Get the send message value
                tmp = this.sendText.Text;
                sendLength = tmp.Length;

                // If the user hasn't inputted the message, the application display a alert message.
                if (sendLength == 0)
                {
                    MessageBox.Show("Input the Message!");
                    return;
                }

                // Convert the string to byte
                for (i = 0; i < sendLength; i++)
                {
                    MsgSendBuff[i] = Convert.ToByte(tmp[i]);
                }

                // If the checkbox is checked, add the CR or LF data to Send data 
                if (this.chk_cr.Checked == true)
                {
                    MsgSendBuff[i] = Convert.ToByte("13");
                    sendLength = sendLength + 1;
                    i++;
                }
                if (this.chk_lf.Checked == true)
                {
                    MsgSendBuff[i] = Convert.ToByte("10");
                    sendLength = sendLength + 1;
                }

                // The socket can continuedly send data because of this function 
                clientsock.BeginSend(MsgSendBuff, 0, sendLength, SocketFlags.None, new AsyncCallback(CallBack_SendMsg), clientsock);
            }
            // If the socket exception is occurrence, the application display a error message.  
            catch (SocketException se)
            {
                MessageBox.Show(se.ErrorCode + ";" + se.Message);
                Environment.Exit(se.ErrorCode);
            }
        }

        // BeginSend's call back function 
        public void CallBack_SendMsg(IAsyncResult ar)
        {
            int length;
            // A variable of decimal message
            string MsgSendStr = null;
            // A variable of hex message
            string MsgSendHexStr = null;

            // get the send message 
            clientsock = (Socket)ar.AsyncState;
            // Get the send message's length
            length = clientsock.EndSend(ar);
            // Encoding the byte to string.
            MsgSendStr = Encoding.Default.GetString(MsgSendBuff, 0, length);

            // Display with hex data
            MsgSendHexStr = ToHexString(MsgSendStr, "Send");

            this.ConnText.AppendText("Sent Data : " + MsgSendHexStr + " | " + MsgSendStr + "\n");
            this.ConnText.ScrollToCaret();
            this.sendText.Clear();
        }

        // Convert the decimal to hex function 
        public string ToHexString(string MsgStr, string chk)
        {
            string MsgHexStr = null;

            if (chk == "Recv")
            {
                for (int i = 0; i < MsgStr.Length; i++)
                {
                    MsgHexStr += MsgRecvBuff[i].ToString("X2") + " ";
                }
            }
            else if (chk == "Send")
            {
                for (int i = 0; i < MsgStr.Length; i++)
                {
                    MsgHexStr += MsgSendBuff[i].ToString("X2") + " ";
                }
            }
            return MsgHexStr;
        }

        // Disconnect button click event
        private void DisconnBtn_Click(object sender, EventArgs e)
        {
            try
            {
                // Shutdown the socket, and dispose the resource.
                clientsock.Shutdown(SocketShutdown.Both);
                this.statText.Text = "Closed\n";

                ConnBtn.Enabled = true;
                DisconnBtn.Enabled = false;
                SendBtn.Enabled = false;

            }
            catch (Exception er)
            {
                MessageBox.Show(er.Message);
            }
        }

    }
}
