﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Threading;

/*
 * 2012.09.28 
 * C# Sample Code  
 * - TCP Asynchronous Server
 * Sollae Systems 
 * Lisa Shin
*/

namespace TCPServerApplication
{

    public partial class Form1 : Form
    {

        public Form1()
        {
            // form intializing 
            InitializeComponent();

            /* variable 
             SendBtn = send button 
             DisconnBtn = disconnect button 
             ConnBtn = connect button 
             ip = IP text box
             port = port text box
             statText = status text box
             ConnText = data text box 
            */
            statText.ReadOnly = true;
            ConnText.ReadOnly = true;
            SendBtn.Enabled = false;
            DisconnBtn.Enabled = false;
            port.Select();          
        }
        
        // A socekt of Listen 
        Socket listener;
        // A socket of server
        Socket serversock;
        // A buffer of receive  
        byte[] MsgRecvBuff = new byte[1024];
        // A buffer of transmitt
        byte[] MsgSendBuff = new byte[1024];

        // Listen button click event
        private void ListenBtn_Click(object sender, EventArgs e)
        {
            string tmp;
            int port;

            this.ListenBtn.Enabled = false;

            // Get the port text value
            tmp = this.port.Text;
            // Convert the string to int
            port = Int32.Parse(tmp);

            // There are IP or port information in the localEP
            IPEndPoint localEP = new IPEndPoint(0000, port);
            // Listener socket memory allocation  
            listener = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP);

            try
            {
                // Bind the port at the socket
                listener.Bind(localEP);
                listener.Listen(10);

                // Wait the signal for connect with client 
                // This application will be run asynchronously by this function.
                // If the client accept the socekt, this function called an AcceptReceiveCallback function.
                listener.BeginAccept(new AsyncCallback(AcceptReceiveCallback), listener);
                this.statText.Text="Listen.. \n";

                this.SendBtn.Enabled = true;
                this.DisconnBtn.Enabled = true;

            }
            catch (SocketException se)
            {
                // If the socket exception is occurrence, the application display a error message.  
                MessageBox.Show(se.ErrorCode + ";" + se.Message);
                Environment.Exit(se.ErrorCode);
            }

            this.DisconnBtn.Enabled = true;
            this.ListenBtn.Enabled = false;
        }

        // BeginAccept()'s call back function 
        public void AcceptReceiveCallback(IAsyncResult ar)
        {

            
            listener = (Socket)ar.AsyncState;

            try
            {
                // If the server socket connect with the client, return the socket
                serversock = listener.EndAccept(ar);

                this.statText.Text = "Connected";
                // If the client transmitted the messages, this function called a Callback_ReceiveMsg function.
                serversock.BeginReceive(MsgRecvBuff, 0, MsgRecvBuff.Length, SocketFlags.None, new AsyncCallback(CallBack_ReceiveMsg), serversock);
            }
            catch (ObjectDisposedException e){}
        }

        // BeginReceive()'s call back fucntion
        public void CallBack_ReceiveMsg(IAsyncResult ar)
        {
            
            int length;
            // A variable of decimal message
            String MsgRecvStr = null;
            // A variable of hex message
            String MsgRecvHexStr = null;

            serversock = (Socket)ar.AsyncState;
            
            try
            {
                // If the socket close, return the 0 value
                length = serversock.EndReceive(ar);

                if (length > 0)
                {

                    // display with decimal data
                    MsgRecvStr = Encoding.Default.GetString(MsgRecvBuff, 0, length);

                    // display with hex data
                    MsgRecvHexStr = ToHexString(MsgRecvStr, "Recv");

                    this.ConnText.AppendText("Received Data : " + MsgRecvHexStr + " | " + MsgRecvStr + "\n");
                    this.ConnText.ScrollToCaret();

                    // The socket can continuedly wait the receive message because of this function
                    serversock.BeginReceive(MsgRecvBuff, 0, MsgRecvBuff.Length, SocketFlags.None, new AsyncCallback(CallBack_ReceiveMsg), serversock);
                }
                else
                {
                    // If the socket close, return the 0 value 
                    // Then, socket close.
                    this.statText.Text = "Closed\n";
                    DisconnBtn.Enabled = false;
                    SendBtn.Enabled = false;
                    serversock.Close();
                    listener.Close();
                    ListenBtn.Enabled = true;
                }
                 
            }
            catch (Exception er){}            
        }
    

        // Send button click event
        private void SendBtn_Click(object sender, EventArgs e)
        {
            String tmp;
            int sendLength;
            int i = 0;

            // Get the send message value
            tmp = this.sendText.Text;
            sendLength = tmp.Length;

            // If the user hasn't inputted the message, the application display a alert message.
            if (sendLength == 0)
            {
                MessageBox.Show("Input the Message!");
                return;
            }

            try
            {
                // Convert the string to byte
                for (i = 0; i < sendLength; i++)
                {
                    MsgSendBuff[i] = Convert.ToByte(tmp[i]);
                }

                // If the checkbox is checked, add the CR or LF data to Send data 
                if (this.chk_cr.Checked == true)
                {
                    MsgSendBuff[i] = Convert.ToByte("13");
                    sendLength = sendLength + 1;
                    i++;
                }
                if (this.chk_lf.Checked == true)
                {
                    MsgSendBuff[i] = Convert.ToByte("10");
                    sendLength = sendLength + 1;          
                }

                // The socket can continuedly send data because of this function 
                serversock.BeginSend(MsgSendBuff, 0, sendLength, SocketFlags.None, new AsyncCallback(CallBack_SendMsg), serversock);
            }
            // If the socket exception is occurrence, the application display a error message.  
            catch (SocketException se)
            {
                MessageBox.Show(se.ErrorCode + ";" + se.Message);
                Environment.Exit(se.ErrorCode);
            }   
        }

        // BeginSend's call back function 
        public void CallBack_SendMsg(IAsyncResult ar)
        {
            int length;
            // A variable of decimal message
            string MsgSendStr = null;
            // A variable of hex message
            string MsgSendHexStr = null;

            serversock = (Socket)ar.AsyncState;
            // Get the send message's length
            length = serversock.EndSend(ar);
           // Encoding the byte to string
            MsgSendStr = Encoding.Default.GetString(MsgSendBuff, 0, length);

            // Display the hex data
            MsgSendHexStr = ToHexString(MsgSendStr, "Send");

            this.ConnText.AppendText("Sent Data : " + MsgSendHexStr + " | " + MsgSendStr + "\n");
            this.ConnText.ScrollToCaret();
            this.sendText.Clear();
        }

        // Convert the decimal to hex function
        public string ToHexString(string MsgStr, string chk)
        {
            string MsgHexStr = null;
            if (chk == "Recv")
            {
                for (int i = 0; i < MsgStr.Length; i++)
                {
                    MsgHexStr += MsgRecvBuff[i].ToString("X2") + " ";
                }
            }
            else if(chk == "Send"){
                for (int i = 0; i < MsgStr.Length; i++)
                {
                    MsgHexStr += MsgSendBuff[i].ToString("X2") + " ";
                }
            }
            return MsgHexStr;
        }

        // Disconnect button click event
        private void DisconnBtn_Click(object sender, EventArgs e)
        {
            try
            {
                if (serversock != null)
                    // Shutdown the socket, and dispose the resource.
                    serversock.Shutdown(SocketShutdown.Both);

                listener.Close();
                this.statText.Text = "Closed\n";

                this.ListenBtn.Enabled = true;
                this.DisconnBtn.Enabled = false;

                
            }
            catch (Exception er)
            {
                MessageBox.Show(er.Message);
            }
        }            

    }

}
